#ifndef __XMLParserAdapter_hpp__
#define __XMLParserAdapter_hpp__

// =================================================================================================
// Copyright 2005 Adobe Systems Incorporated
// All Rights Reserved.
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
// of the Adobe license agreement accompanying it.
// =================================================================================================

#include "XMP_Environment.h"	// ! Must be the first #include!
#include "XMP_Impl.hpp"

// =================================================================================================
// Abstract base class for XML parser adapters used by the XMP toolkit.
// =================================================================================================

enum { kXMLParseTailSize = 8 };

class XMLParserAdapter {
public:

	XMLParserAdapter()
		: tree(0,"",kRootNode), rootNode(0), rootCount(0),
		  charEncoding(XMP_OptionBits(-1)), partialUniSize(0), UniConverter(0)
	{
		prevTail[0] = 0;
		#if XMP_DebugBuild
			parseLog = 0;
		#endif
	};

	virtual ~XMLParserAdapter() {};
	
	virtual void ParseBuffer ( void * buffer, size_t length, bool last ) = 0;

	XML_Node		tree;
	XML_NodeVector	parseStack;
	XML_Node *		rootNode;
	size_t			rootCount;

	XMP_OptionBits	charEncoding;
	size_t			partialUniSize;		// The number of bytes in partialUniChar.
	unsigned char	partialUniChar[4];	// A partial UTF-16 or UTF-32 character.
	void *          UniConverter;		// A UTF-nn to UTF-8 conversion function.
	unsigned char	prevTail[kXMLParseTailSize];	// A null terminated partial UTF-8 escape sequence.
	
	#if XMP_DebugBuild
		FILE * parseLog;
	#endif

};

// =================================================================================================

#endif	// __XMLParserAdapter_hpp__
